;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

;; Place your private configuration here! Remember, you do not need to run 'doom
;; sync' after modifying this file!


;; Some functionality uses this to identify you, e.g. GPG configuration, email
;; clients, file templates and snippets.
(setq user-full-name "Joseph Ferano"
      user-mail-address "joseph@ctech.com")

;; Doom exposes five (optional) variables for controlling fonts in Doom. Here
;; are the three important ones:
;;
;; + `doom-font'
;; + `doom-variable-pitch-font'
;; + `doom-big-font' -- used for `doom-big-font-mode'; use this for
;;   presentations or streaming.
;;
;; They all accept either a font-spec, font string ("Input Mono-12"), or xlfd
;; font string. You generally only need these two:
;; (setq doom-font (font-spec :family "monospace" :size 12 :weight 'semi-light)
;;       doom-variable-pitch-font (font-spec :family "sans" :size 13))

(setq doom-font (font-spec :family "Fira Code Nerd Font" :size 15))

;; There are two ways to load a theme. Both assume the theme is installed and
;; available. You can either set `doom-theme' or manually load a theme with the
;; `load-theme' function. This is the default:
(setq doom-theme 'doom-vibrant)
;; Also looks nice doom-palenight

;; If you use `org' and don't want your org files in the default location below,
;; change `org-directory'. It must be set before org loads!
(setq org-directory "~/org/")

;; This determines the style of line numbers in effect. If set to `nil', line
;; numbers are disabled. For relative line numbers, set this to `relative'.
(setq display-line-numbers-type 'relative)


;; Here are some additional functions/macros that could help you configure Doom:
;;
;; - `load!' for loading external *.el files relative to this one
;; - `use-package!' for configuring packages
;; - `after!' for running code after a package has loaded
;; - `add-load-path!' for adding directories to the `load-path', relative to
;;   this file. Emacs searches the `load-path' when you load packages with
;;   `require' or `use-package'.
;; - `map!' for binding new keys
;;
;; To get information about any of these functions/macros, move the cursor over
;; the highlighted symbol at press 'K' (non-evil users must press 'C-c c k').
;; This will open documentation for it, including demos of how they are used.
;;
;; You can also try 'gd' (or 'C-c c d') to jump to their definition and see how
;; they are implemented.

(setq default-directory "~/")

(setq which-key-idle-delay 0.4)

(setq dired-listing-switches "-ahlgo -v --group-directories-first")

(setq recentf-max-saved-items 1000)

(defun joe/dired-same-buffer ()
    "In Dired, visit this file or directory in another window."
    (interactive)
    (find-alternate-file ".."))

(defun joe/dired-open-with-wsl-open ()
  "Open file under cursor with wsl-open which should launch it on the windows side"
  (interactive)
  (shell-command (format "wsl-open '%s'" (dired-get-filename))))

(map!
    (:n "-" #'dired-jump)
    (:n "C-h" #'dired-jump)
    (:after dired
    :map dired-mode-map
        :n "o" #'joe/dired-open-with-wsl-open
        :n "s" #'dired-sort-toggle-or-edit
        :n "-" #'joe/dired-same-buffer
        :n "RET" #'dired-find-alternate-file)
    (:after avy
        :n "q" #'avy-goto-word-0))

(require 'avy)
(defvar avy-map (make-sparse-keymap))

(use-package! avy
  :init
  (setq avy-keys '(?a ?s ?d ?f ?w ?e ?r ?u ?i ?o ?h ?j ?k ?l ?x ?c ?m ?z ?p ?q ?t ?y ?b ?n))
  (setq avy-all-windows nil)
  (setq avy-background t))
